module ActiveRedis
  
  def self.included(base)
    base.send(:include, Paperclip::Resource)
    base.send(:include, DataMapper::Resource)
    base.send(:include, Sunspot::ActiveRedis)
    base.send(:extend, ActiveRedisClassMethods)
    base.before :save, :use_param_attribute
    base.after :save, :add_to_index
    base.after :destroy, :remove_from_index
    base.class_eval do
      alias_method :to_param, :new_param
    end
  end
  
  def add_to_index
    if self.class.is_using_sunspot
      Sunspot.index!(self)
    end
  end
  
  def remove_from_index
    if self.class.is_using_sunspot
      Sunspot.remove!(self)
    end
  end
  
  def use_param_attribute
    if self.class.param_attribute != nil
      self.param_name = self.send(self.class.param_attribute).parameterize
    end
  end
  
  def new_param
    self.class.param_attribute == nil ? id.to_s : self.param_name.to_s
  end
  
  def self.find_by_param_name(param_name)
    first(:param_name => param_name)
  end
  
  def find(args)
    get(args)
  end

  def self.model_name
    @_model_name ||= ActiveModel::Name.new(self)
  end
  
  def to_key
    new? ? nil : [id]
  end
  
  def persisted?
    new? ? false : true
  end
  
  def self.parse_date_time(hash)
    datetime = DateTime.new(hash["datetime(1i)"].to_i, hash['datetime(2i)'].to_i, hash['datetime(3i)'].to_i, hash["datetime(4i)"].to_i, hash["datetime(5i)"].to_i)
  end
  
  def self.parse_date(hash)
    date = DateTime.new(hash["date(1i)"].to_i, hash['date(2i)'].to_i, hash['date(3i)'].to_i, 0, 0)
    return date.to_date
  end
  
  def to_s
    self.id
  end
  
  module  ActiveRedisClassMethods
    attr_accessor :param_attribute, :is_using_sunspot
    
    def param_attribute_property(property)
      @param_attribute = property
    end
    
    def add_sunspot_hooks
      @is_using_sunspot = true
    end
  end
  
end